/***********************************************************************
 * File: utf8.h
 * 	(Extracted from
 *	   "$Id: utf8.h,v 1.1 2008/10/05 15:55:33 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C header defines prototyping for the UTF-8 character decoder
 *  and encoder.
 *
 * Copyright (C) 2008 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/


#ifndef _UTF8_H_ /* { */
#define _UTF8_H_
#ifdef __cplusplus
extern "C" {
#endif

/*
 * including the system definitions
 */
#include <unistd.h>
#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <limits.h>

typedef enum Sanitize_for_UTF8_e {
	  Sanitize_for_UTF8_e_EUCJP	/* the source is encoded in EUC-JP. */
	, Sanitize_for_UTF8_e_SJIS	/* the source is encoded in SJIS. */
	, Sanitize_for_UTF8_e_N		/* number of the source encoding. */
	} Sanitize_for_UTF8_e;

/* this function sanitizes a string (safer UTF-8 encoding)
 * EINVAL means "illegal arguments".
 */
extern unsigned char *sanitize_for_utf8(Sanitize_for_UTF8_e	src_encode
										, size_t				length
										, unsigned char			*str
										, char					replace_code
										);

/* to_utf8()'s `how' argument is zero or logical-or the followings...
 * (default, to_utf8() converts Yen-sign (\x5c)  to UTF-8 "\xc2\xa5 U+00A5",
 *  and convert tilde (0x7e) to UTF-8 "\x7e")
 */
#define Sanitize_for_UTF8_Yen_sign_must_be_U005C	(1UL)
#define Sanitize_for_UTF8_Tilde_must_be_U203E		(2UL)
#define Sanitize_for_UTF8_Default_how				(0UL)

/* convert the EUC-JP or SJIS encoded string to UTF-8 encoded.
 * EINVAL means "illegal arguments".
 * EPERM means "system not support EUC-JP, SJIS or UTF-8 encode".
 * EILSEQ means "`inbuf' contains illegal character".
 */
extern unsigned char *to_utf8(Sanitize_for_UTF8_e	inbuf_encode
							, unsigned char			*inbuf
							, size_t				inbuf_length
							, char					replace_code
							, unsigned char			*outbuf
							, size_t				outbuf_size
							, char					padding_code
							, unsigned long			how
							);

/* convert the UTF-8 encoded string to EUC-JP or SJIS encoded.
 * EINVAL means "illegal arguments".
 * EPERM means "system not support EUC-JP, SJIS or UTF-8 encode".
 * EILSEQ means "`inbuf' contains illegal character".
 */
extern unsigned char *from_utf8(Sanitize_for_UTF8_e	outbuf_encode
							, unsigned char			*inbuf
							, size_t				inbuf_length
							, unsigned char			*outbuf
							, size_t				outbuf_size
							, char					padding_code
							);

#ifdef __cplusplus
}
#endif
#endif /* } (!defined(_UTF8_H_) */
