/***********************************************************************
 * File: utf8_test.c
 * 	(Extracted from
 *	   "$Id: utf8_test.c,v 1.2 2008/10/06 13:04:56 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C source tests the `utf8.c' API-s.
 *
 * Copyright (C) 2008 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/


/* include this library definitions */
#include "utf8.h"

/* this function sanitizes a string (safer UTF-8 encoding) */
int main(int ac, char **av)
{
	static unsigned char	*utf8_redundant_str = (unsigned char *)
		"{{{ source is encoded in UTF-8 redundant:\n"
		"UTF-8 period 0x2e: "		"\x2e"
									"\xc0\xae"
									"\xe0\x80\xae"
									"\xf0\x80\x80\xae"
									"\xf8\x80\x80\x80\xae"
									"\xfc\x80\x80\x80\x80\xae"		"\n"
		"UTF-8 backsrash 0x5c: "	"\x5c"
									"\xc1\x9c"
									"\xe0\x81\x9c"
									"\xf0\x80\x81\x9c"
									"\xf8\x80\x80\x81\x9c"
									"\xfc\x80\x80\x80\x81\x9c"		"\n"
		"UTF-8 Yen sign 0x5c: "		"\x5c"
									"\xc2\xa5"
									"\xe0\x82\xa5"
									"\xf0\x80\x82\xa5"
									"\xf8\x80\x80\x82\xa5"
									"\xfc\x80\x80\x80\x82\xa5"		"\n"
		"UTF-8 tilde 0x7e: "		"\x7e"
									"\xc1\xbe"
									"\xe2\x80\xbe"
									"\xf0\x82\x80\xbe"
									"\xf8\x80\x82\x80\xbe"
									"\xfc\x80\x80\x82\x80\xbe"		"\n"
		"}}}\n"
	, *euc_string = (unsigned char *)
		"{{{ source is encoded in EUC-JP:\n"
		"JIS X 0201 (ASCII 20 to 7d): "	"\x20"		"-" "\x7d"		"\n"
		"JIS X 0201 (ASCII signs): " "\x21,\x22,\x23,\x24,\x25,\x26,\x27"
									 ",\x28,\x29,\x2a,\x2b,\x2c,\x2d,\x2e"
									 ",\x2f,\x3a,\x3b,\x3c,\x3d,\x3e,\x3f"
									 ",\x40,\x5b,\x5d,\x5e,\x5f,\x60,\x7b"
									 ",\x7c,\x7d"				"\n"
		"JIS X 0201 (ASCII 5c:backslash/yen-sign): "	"\x5c"		"\n"
		"JIS X 0201 (ASCII 7e:tilde): "					"\x7e"		"\n"
		"JIS X 0201 (ASCII 7f:DEL): "					"\x7f"		"\n"
		"JIS X 0201 (Kana:garbled): "	"\x8e\x21"	"-" "\x8e\x5f"	"\n"
		"JIS X 0208 (01-01 to 02-14): "	"\xa1\xa1"	"-" "\xa2\xae"	"\n"
		"JIS X 0208 (02-15 to 02-25:garbled): "
										"\xa2\xaf"	"-" "\xa2\xb9"	"\n"
		"JIS X 0208 (02-26 to 02-33): "	"\xa2\xba"	"-" "\xa2\xc1"	"\n"
		"JIS X 0208 (02-34 to 02-41:garbled): "
										"\xa2\xc2"	"-" "\xa2\xc9"	"\n"
		"JIS X 0208 (02-42 to 02-48): "	"\xa2\xca"	"-" "\xa2\xd0"	"\n"
		"JIS X 0208 (02-49 to 02-59:garbled): "
										"\xa2\xd1"	"-" "\xa2\xdb"	"\n"
		"JIS X 0208 (02-60 to 02-74): "	"\xa2\xdc"	"-" "\xa2\xea"	"\n"
		"JIS X 0208 (02-75 to 02-81:garbled): "
										"\xa2\xeb"	"-" "\xa2\xf1"	"\n"
		"JIS X 0208 (02-82 to 02-89): "	"\xa2\xf2"	"-" "\xa2\xf9"	"\n"
		"JIS X 0208 (02-90 to 02-93:garbled): "
										"\xa2\xfa"	"-" "\xa2\xfd"	"\n"
		"JIS X 0208 (02-94): "			"\xa2\xfe"					"\n"
		"JIS X 0208 (03-01 to 03-15:garbled): "
										"\xa3\xa1"	"-" "\xa3\xaf"	"\n"
		"JIS X 0208 (03-16 to 03-25): "	"\xa3\xb0"	"-" "\xa3\xb9"	"\n"
		"JIS X 0208 (03-26 to 03-32:garbled): "
										"\xa3\xba"	"-" "\xa3\xc0"	"\n"
		"JIS X 0208 (03-33 to 03-58): "	"\xa3\xc1"	"-" "\xa3\xda"	"\n"
		"JIS X 0208 (03-59 to 03-64:garbled): "
										"\xa3\xdb"	"-" "\xa3\xe0"	"\n"
		"JIS X 0208 (03-65 to 03-90): "	"\xa3\xe1"	"-" "\xa3\xfa"	"\n"
		"JIS X 0208 (03-91 to 03-94:garbled): "
										"\xa3\xfb"	"-" "\xa3\xfe"	"\n"
		"JIS X 0208 (04-01 to 04-83): "	"\xa4\xa1"	"-" "\xa4\xf3"	"\n"
		"JIS X 0208 (04-84 to 04-94:garbled): "
										"\xa4\xf4"	"-" "\xa4\xfe"	"\n"
		"JIS X 0208 (05-01 to 05-86): "	"\xa5\xa1"	"-" "\xa5\xf6"	"\n"
		"JIS X 0208 (05-87 to 05-94:garbled): "
										"\xa5\xf7"	"-" "\xa5\xfe"	"\n"
		"JIS X 0208 (06-01 to 06-24): "	"\xa6\xa1"	"-" "\xa6\xb8"	"\n"
		"JIS X 0208 (06-25 to 06-32:garbled): "
										"\xa6\xb9"	"-" "\xa6\xc0"	"\n"
		"JIS X 0208 (06-33 to 06-56): "	"\xa6\xc1"	"-" "\xa6\xd8"	"\n"
		"JIS X 0208 (06-57 to 06-94:garbled): "
										"\xa6\xd9"	"-" "\xa6\xfe"	"\n"
		"JIS X 0208 (07-01 to 07-33): "	"\xa7\xa1"	"-" "\xa7\xc1"	"\n"
		"JIS X 0208 (07-34 to 07-47:garbled): "
										"\xa7\xc2"	"-" "\xa7\xd0"	"\n"
		"JIS X 0208 (07-48 to 07-81): "	"\xa7\xd1"	"-" "\xa7\xf1"	"\n"
		"JIS X 0208 (07-82 to 07-94:garbled): "
										"\xa7\xf2"	"-" "\xa7\xfe"	"\n"
		"JIS X 0208 (08-01 to 08-32): "	"\xa8\xa1"	"-" "\xa8\xc0"	"\n"
		"JIS X 0208 (08-33 to 08-94:garbled): "
										"\xa8\xc1"	"-" "\xa8\xfe"	"\n"
		"JIS X 0208 (09-01 to 09-94:garbled): "
										"\xa9\xa1"	"-" "\xa9\xfe"	"\n"
		"JIS X 0208 (10-01 to 10-94:garbled): "
										"\xaa\xa1"	"-" "\xaa\xfe"	"\n"
		"JIS X 0208 (11-01 to 11-94:garbled): "
										"\xab\xa1"	"-" "\xab\xfe"	"\n"
		"JIS X 0208 (12-01 to 12-94:garbled): "
										"\xac\xa1"	"-" "\xac\xfe"	"\n"
		"JIS X 0208 (13-01 to 13-30 (circled and Roman digits):garbled): "
										"\xad\xa1"	"-" "\xad\xbe"	"\n"
		"JIS X 0208 (13-31:garbled): "	"\xad\xbf"					"\n"
		"JIS X 0208 (13-32 to 13-54 (units):garbled): "
										"\xad\xc0"	"-" "\xad\xd6"	"\n"
		"JIS X 0208 (13-55 to 13-62:garbled): "
										"\xad\xd7"	"-" "\xad\xde"	"\n"
		"JIS X 0208 (13-63 to 13-92 (name of era, etc):garbled): "
										"\xad\xdf"	"-" "\xad\xfc"	"\n"
		"JIS X 0208 (13-93 to 13-94:garbled): "
										"\xad\xfd"	"-" "\xad\xfe"	"\n"
		"JIS X 0208 (14-01 to 14-94:garbled): "
										"\xae\xa1"	"-" "\xae\xfe"	"\n"
		"JIS X 0208 (15-01 to 15-94:garbled): "
										"\xaf\xa1"	"-" "\xaf\xfe"	"\n"
		"JIS X 0208 (16-01 to 47-51): "	"\xb0\xa1"	"-" "\xcf\xd3"	"\n"
		"JIS X 0208 (47-52 to 47-94:garbled): "
										"\xcf\xd4"	"-" "\xcf\xfe"	"\n"
		"JIS X 0208 (48-01 to 84-06): "	"\xd0\xa1"	"-" "\xf4\xa6"	"\n"
		"JIS X 0208 (84-07 to 84-94:garbled): "
										"\xf4\xa7"	"-" "\xf4\xfe"	"\n"
		"JIS X 0208 (85-01 to 85-94:garbled): "
										"\xf5\xa1"	"-" "\xf5\xfe"	"\n"
		"JIS X 0208 (86-01 to 86-94:garbled): "
										"\xf6\xa1"	"-" "\xf6\xfe"	"\n"
		"JIS X 0208 (87-01 to 87-94:garbled): "
										"\xf7\xa1"	"-" "\xf7\xfe"	"\n"
		"JIS X 0208 (88-01 to 88-94:garbled): "
										"\xf8\xa1"	"-" "\xf8\xfe"	"\n"
		"JIS X 0208 (89-01 to 89-94 (JIS defined):garbled): "
										"\xf9\xa1"	"-" "\xf9\xfe"	"\n"
		"JIS X 0208 (90-01 to 90-94 (JIS defined):garbled): "
										"\xfa\xa1"	"-" "\xfa\xfe"	"\n"
		"JIS X 0208 (91-01 to 91-94 (JIS defined):garbled): "
										"\xfb\xa1"	"-" "\xfb\xfe"	"\n"
		"JIS X 0208 (92-01 to 92-94 (JIS defined):garbled): "
										"\xfc\xa1"	"-" "\xfc\xfe"	"\n"
		"JIS X 0208 (93-01 to 93-94:garbled): "
										"\xfd\xa1"	"-" "\xfd\xfe"	"\n"
		"JIS X 0208 (94-01 to 94-94:garbled): "
										"\xfe\xa1"	"-" "\xfe\xfe"	"\n"
		"}}}\n"
	, *sjis_string = (unsigned char *)
		"{{{ source is encoded in SJIS:\n"
		"JIS X 0201 (ASCII 20 to 7d): "	"\x20"		"-" "\x7d"		"\n"
		"JIS X 0201 (ASCII signs): " "\x21,\x22,\x23,\x24,\x25,\x26,\x27"
									 ",\x28,\x29,\x2a,\x2b,\x2c,\x2d,\x2e"
									 ",\x2f,\x3a,\x3b,\x3c,\x3d,\x3e,\x3f"
									 ",\x40,\x5b,\x5d,\x5e,\x5f,\x60,\x7b"
									 ",\x7c,\x7d"				"\n"
		"JIS X 0201 (ASCII 5c:backslash/yen-sign): "	"\x5c"		"\n"
		"JIS X 0201 (ASCII 7e:tilde): "					"\x7e"		"\n"
		"JIS X 0201 (ASCII 7f:DEL): "					"\x7f"		"\n"
		"JIS X 0201 (Kana:garbled): "	"\xa1"		"-" "\xdf"		"\n"
		"JIS X 0208 (01-01 to 02-14): "	"\x81\x40"	"-" "\x81\xac"	"\n"
		"JIS X 0208 (02-15 to 02-25:garbled): "
										"\x81\xad"	"-" "\x81\xb7"	"\n"
		"JIS X 0208 (02-26 to 02-33): "	"\x81\xb8"	"-"	"\x81\xbf"	"\n"
		"JIS X 0208 (02-34 to 02-41:garbled): "
										"\x81\xc0"	"-" "\x81\xc7"	"\n"
		"JIS X 0208 (02-42 to 02-48): "	"\x81\xc8" 	"-"	"\x81\xce"	"\n"
		"JIS X 0208 (02-49 to 02-59:garbled): "
										"\x81\xcf"	"-" "\x81\xd9"	"\n"
		"JIS X 0208 (02-60 to 02-74): "	"\x81\xda" 	"-"	"\x81\xe8"	"\n"
		"JIS X 0208 (02-75 to 02-81:garbled): "
										"\x81\xe9"	"-" "\x81\xef"	"\n"
		"JIS X 0208 (02-82 to 02-89): "	"\x81\xf0" 	"-"	"\x81\xf7"	"\n"
		"JIS X 0208 (02-90 to 02-93:garbled): "
										"\x81\xf8"	"-" "\x81\xfb"	"\n"
		"JIS X 0208 (02-94): "			"\x81\xfc"					"\n"
		"JIS X 0208 (03-01 to 03-15:garbled): "
										"\x82\x40"	"-" "\x82\x4e"	"\n"
		"JIS X 0208 (03-16 to 03-25): "	"\x82\x4f" 	"-"	"\x82\x58"	"\n"
		"JIS X 0208 (03-26 to 03-32:garbled): "
										"\x82\x59"	"-" "\x82\x5f"	"\n"
		"JIS X 0208 (03-33 to 03-58): "	"\x82\x60" 	"-"	"\x82\x79"	"\n"
		"JIS X 0208 (03-59 to 03-64:garbled): "
										"\x82\x7a"	"-" "\x82\x80"	"\n"
		"JIS X 0208 (03-65 to 03-90): "	"\x82\x81" 	"-"	"\x82\x9a"	"\n"
		"JIS X 0208 (03-91 to 03-94:garbled): "
										"\x82\x9b"	"-" "\x82\x9e"	"\n"
		"JIS X 0208 (04-01 to 04-83): "	"\x82\x9f" 	"-"	"\x82\xf1"	"\n"
		"JIS X 0208 (04-84 to 04-94:garbled): "
										"\x82\xf2"	"-" "\x82\xfc"	"\n"
		"JIS X 0208 (05-01 to 05-86): "	"\x83\x40" 	"-"	"\x83\x96"	"\n"
		"JIS X 0208 (05-87 to 05-94:garbled): "
										"\x83\x97"	"-" "\x83\x9e"	"\n"
		"JIS X 0208 (06-01 to 06-24): "	"\x83\x9f" 	"-"	"\x83\xb6"	"\n"
		"JIS X 0208 (06-25 to 06-32:garbled): "
										"\x83\xb7"	"-" "\x83\xbe"	"\n"
		"JIS X 0208 (06-33 to 06-56): "	"\x83\xbf" 	"-"	"\x83\xd6"	"\n"
		"JIS X 0208 (06-57 to 06-94:garbled): "
										"\x83\xd7"	"-" "\x83\xfc"	"\n"
		"JIS X 0208 (07-01 to 07-33): "	"\x84\x40" 	"-"	"\x84\x60"	"\n"
		"JIS X 0208 (07-34 to 07-47:garbled): "
										"\x84\x61"	"-" "\x84\x6f"	"\n"
		"JIS X 0208 (07-48 to 07-81): "	"\x84\x70" 	"-"	"\x84\x91"	"\n"
		"JIS X 0208 (07-82 to 07-94:garbled): "
										"\x84\x92"	"-" "\x84\x9e"	"\n"
		"JIS X 0208 (08-01 to 08-32): "	"\x84\x9f" 	"-"	"\x84\xbe"	"\n"
		"JIS X 0208 (08-33 to 08-94:garbled): "
										"\x84\xbf"	"-" "\x84\xfc"	"\n"
		"JIS X 0208 (09-01 to 09-94:garbled): "
										"\x85\x40"	"-" "\x85\x9e"	"\n"
		"JIS X 0208 (10-01 to 10-94:garbled): "
										"\x85\x9f"	"-" "\x85\xfc"	"\n"
		"JIS X 0208 (11-01 to 11-94:garbled): "
										"\x86\x40"	"-" "\x86\x9e"	"\n"
		"JIS X 0208 (12-01 to 12-94:garbled): "
										"\x86\x9f"	"-" "\x86\xfc"	"\n"
		"JIS X 0208 (13-01 to 13-30 (circled and Roman digits):garbled): "
										"\x87\x40"	"-" "\x87\x5d"	"\n"
		"JIS X 0208 (13-31:garbled): "	"\x87\x5e"					"\n"
		"JIS X 0208 (13-32 to 13-54 (units):garbled): "
										"\x87\x5f"	"-"	"\x87\x75"	"\n"
		"JIS X 0208 (13-55 to 13-62:garbled): "
										"\x87\x76"	"-"	"\x87\x7d"	"\n"
		"JIS X 0208 (13-63 to 13-92 (name of era, etc):garbled): "
										"\x87\x7e"	"-"	"\x87\x9c"	"\n"
		"JIS X 0208 (13-93 to 13-94:garbled): "
										"\x87\x9d"	"-"	"\x87\x9e"	"\n"
		"JIS X 0208 (14-01 to 14-94:garbled): "
										"\x87\x9f"	"-" "\x87\xfc"	"\n"
		"JIS X 0208 (15-01 to 15-94:garbled): "
										"\x88\x40"	"-" "\x88\x9e"	"\n"
		"JIS X 0208 (16-01 to 47-51): "	"\x88\x9f" 	"-"	"\x98\x72"	"\n"
		"JIS X 0208 (47-52 to 47-94:garbled): "
										"\x98\x73"	"-" "\x98\x9e"	"\n"
		"JIS X 0208 (48-01 to 84-06): "	"\x98\x9f" 	"-"	"\xea\xa4"	"\n"
		"JIS X 0208 (84-07 to 84-94:garbled): "
										"\xea\xa5"	"-" "\xea\xfc"	"\n"
		"JIS X 0208 (85-01 to 85-94:garbled): "
										"\xeb\x40"	"-" "\xeb\x9e"	"\n"
		"JIS X 0208 (86-01 to 86-94:garbled): "
										"\xeb\x9f"	"-" "\xeb\xfc"	"\n"
		"JIS X 0208 (87-01 to 87-94:garbled): "
										"\xec\x40"	"-" "\xec\x9e"	"\n"
		"JIS X 0208 (88-01 to 88-94:garbled): "
										"\xec\x9f"	"-" "\xec\xfc"	"\n"
		"JIS X 0208 (89-01 to 89-94 (JIS defined):garbled): "
										"\xed\x40"	"-" "\xed\x9e"	"\n"
		"JIS X 0208 (90-01 to 90-94 (JIS defined):garbled): "
										"\xed\x9f"	"-" "\xed\xfc"	"\n"
		"JIS X 0208 (91-01 to 91-94 (JIS defined):garbled): "
										"\xee\x40"	"-" "\xee\x9e"	"\n"
		"JIS X 0208 (92-01 to 92-94 (JIS defined):garbled): "
										"\xee\x9f"	"-" "\xee\xfc"	"\n"
		"JIS X 0208 (93-01 to 93-94:garbled): "
										"\xef\x40"	"-" "\xef\x9e"	"\n"
		"JIS X 0208 (94-01 to 94-94:garbled): "
										"\xef\x9f"	"-" "\xef\xfc"	"\n"
		"}}}\n"
		;
	Sanitize_for_UTF8_e	str_encoding = Sanitize_for_UTF8_e_N;
	unsigned long		how = Sanitize_for_UTF8_Default_how;
	unsigned char		*str = NULL, *utf8str = NULL;
	size_t				length = 0, utf8str_size = 0;
	FILE				*fp = NULL;
	char				*fname = NULL, *errmsg = NULL
						, replace_code = '@'
						, padding_code = '$'
						;
	int					sv_errno = 0;

	/* dump UTF-8 redundant code text */
	fname	= "utf8.redundant.txt";
	if ((fp = fopen(fname, "w")) == NULL) {
		sv_errno = errno; errmsg = strerror(sv_errno)
		; fprintf(stderr
			, "%s:L%d:%s(): Error! can't open %s (errno=%d: %s)\n"
			, __FILE__, __LINE__, __func__
			, fname, sv_errno, errmsg
			); (void)fflush(stderr)
			;
		exit(EXIT_FAILURE);	/* Fatal exit */
	}
	fprintf(fp, "%s", (char *)utf8_redundant_str); (void)fflush(fp)
	; (void)fclose(fp)
	;
	fprintf(stderr, "%s:L%d:%s(): dumped UTF-8 redundant source data to "
					"\"%s\".\n"
					, __FILE__, __LINE__, __func__
					, fname
					); (void)fflush(stderr)
					;

	if (((ac == 1)
	|| strcmp(av[1], "--euc") == 0) || strcmp(av[1], "-e") == 0) {
		unsigned char		*result = NULL;

		/* invoke from_utf8() (UTF-8 redundant code) */
		str_encoding	= Sanitize_for_UTF8_e_EUCJP
		; utf8str_size	= strlen((char *)utf8_redundant_str)
		; length		= utf8str_size
		; if ((str = malloc((length + 1))) == NULL) {
			sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr
				, "%s:L%d:%s: Error! can't allocate str buffer "
					"(length+1=%lu)"
					"(errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, (unsigned long)(length + 1)
				, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		memset(str, (int)'\0', length)
		; padding_code	= '\0'
		; errno			= 0
		;
		fprintf(stderr, "%s:L%d:%s(): from_utf8(outbuf_encode=%d, "
						"inbuf@%p, inbuf_length=%lu, outbuf@%p, "
						"outbuf_size=%lu, padding_code='%c'(0x%02x)) "
						"returns "
						, __FILE__, __LINE__, __func__, str_encoding
						, utf8_redundant_str, (unsigned long)utf8str_size
						, str, (unsigned long)length
						, padding_code, (unsigned)padding_code
						); (void)fflush(stderr)
		; result	= from_utf8(str_encoding
								, utf8_redundant_str
								, utf8str_size
								, str
								, length
								, padding_code
								)
		; sv_errno = errno; errmsg = strerror(sv_errno)
		; fprintf(stderr, "%p (len=%lu) (errno=%d: %s)\n"
						, result, (unsigned long)strlen((char *)result)
						, sv_errno, errmsg
						); (void)fflush(stderr)
						;

		/* dump converted UTF-8 redundant code */
		fname	= "euc.from_utf8.redundant.txt";
		if ((fp = fopen(fname, "w")) == NULL) {
			sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr
				, "%s:L%d:%s(): Error! can't open %s (errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, fname, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		fprintf(fp, "%s", (char *)str); (void)fflush(fp)
		; (void)fclose(fp)
		;
		fprintf(stderr, "%s:L%d:%s(): dumped converted UTF-8 redundant to "
						"EUC-JP in \"%s\".\n"
						, __FILE__, __LINE__, __func__
						, fname
						); (void)fflush(stderr)
						;

		/* release converted EUC-JP buffer */
		free(str); str = NULL;

		/* dump source text */
		fname	= "euc.src.txt";
		if ((fp = fopen(fname, "w")) == NULL) {
			sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr
				, "%s:L%d:%s(): Error! can't open %s (errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, fname, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		fprintf(fp, "%s", (char *)euc_string); (void)fflush(fp)
		; (void)fclose(fp)
		;
		fprintf(stderr, "%s:L%d:%s(): dumped EUC-JP source data to "
						"\"%s\".\n"
						, __FILE__, __LINE__, __func__
						, fname
						); (void)fflush(stderr)
						;

		/* invoke `sanitize_for_utf8()' */
		length	= strlen((char *)euc_string) + 1;
		if ((str = malloc((length + 1))) == NULL) {
			sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr
				, "%s:L%d:%s: Error! can't allocate euc_string buffer "
					"(length+1=%lu)"
					"(errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, (unsigned long)(length + 1)
				, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		str_encoding	= Sanitize_for_UTF8_e_EUCJP
		; length			= strlen((char *)euc_string)
		; strcpy(memset(str, (int)'\0', (length + 1)), (char *)euc_string)
		; replace_code	= '@'
		; fprintf(stderr, "%s:L%d:%s(): sanitize_for_utf8("
						"str_encoding=%d"
						", length=%lu (strlen(euc_string)=%lu)"
						", str@%p"
						", replace_code='%c'(=0x%02x)"
						") returns "
						, __FILE__, __LINE__, __func__
						, (int)str_encoding
						, (unsigned long)length
						,		(unsigned long)strlen((char *)euc_string)
						, (void *)str
						, replace_code, (unsigned)replace_code
						); (void)fflush(stderr)
		;
		errno	= 0
		; result	= sanitize_for_utf8(str_encoding
										, length
										, str
										, replace_code
										)
		; sv_errno	= errno; errmsg = strerror(sv_errno)
		;
		fprintf(stderr, "@%p(len=%lu) (errno=%d: %s).\n"
						, result, (unsigned long)strlen((char *)result)
						, sv_errno, errmsg
						); (void)fflush(stderr)
						;

		/* dump output of `sanitize_for_utf8()' */
		fname	= "euc.sanitized.txt";
		if ((fp = fopen(fname, "w")) == NULL) {
			sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr
				, "%s:L%d:%s(): Error! can't open %s (errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, fname, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		fprintf(fp, "%s", (char *)str); (void)fflush(fp)
		; (void)fclose(fp)
		;
		fprintf(stderr, "%s:L%d:%s(): dumped EUC-JP sanitized data to "
						"\"%s\".\n"
						, __FILE__, __LINE__, __func__
						, fname
						); (void)fflush(stderr)
						;

		/* invoke to_utf8() */
		str_encoding	= Sanitize_for_UTF8_e_EUCJP
		; length		= strlen((char *)euc_string)
		; memcpy(str,	euc_string, length)
		; replace_code	= '@'
		; utf8str_size	= (2 * length)
		;
		if ((utf8str 	= malloc((utf8str_size + 1))) == NULL) {
			sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr
				, "%s:L%d:%s: Error! can't allocate utf8str buffer "
					"(utf8str_size+1=%lu)"
					"(errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, (unsigned long)(utf8str_size + 1)
				, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		memset(utf8str, (int)'\0', (utf8str_size + 1))
		; utf8str_size	= length
		; padding_code	= '\0'
		; how			= Sanitize_for_UTF8_Default_how
		; errno			= 0;
		;
		; fprintf(stderr,	"%s:L%d:%s(): to_utf8("
							"inbuf_encode=%d, inbuf@%p, inbuf_length=%lu"
							", replace_code='%c'(=0x%02x)"
							", outbuf@%p, outbuf_size=%lu"
							", padding_code='%c'(=0x%02x)"
							", how=0x%lx"
							") returns "
							, __FILE__, __LINE__, __func__
							, str_encoding, str, (unsigned long)length
							, replace_code, (unsigned)replace_code
							, utf8str, (unsigned long)utf8str_size
							, padding_code, (unsigned)padding_code
							, how
							); (void)fflush(stderr)
		; result = to_utf8(str_encoding
							, str
							, length
							, replace_code
							, utf8str
							, utf8str_size
							, padding_code
							, how
							)
		; sv_errno = errno; errmsg = strerror(sv_errno)
		; fprintf(stderr, "@%p(len=%lu) (errno=%d: %s)\n"
							, result, (unsigned long)strlen((char *)result)
							, errno, errmsg
							); (void)fflush(stderr)
							;

		/* dump output of `to_utf8()' (utf8str_size equals length) */
		fname	= "euc.to_utf8.same_buff_size.txt";
		if ((fp = fopen(fname, "w")) == NULL) {
			sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr
				, "%s:L%d:%s(): Error! can't open %s (errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, fname, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		fprintf(fp, "%s", (char *)utf8str); (void)fflush(fp)
		; (void)fclose(fp)
		;
		fprintf(stderr, "%s:L%d:%s(): dumped EUC-JP `to_utf8( same buff "
						"size)' data to \"%s\".\n"
						, __FILE__, __LINE__, __func__
						, fname
						); (void)fflush(stderr)
						;

		str_encoding	= Sanitize_for_UTF8_e_EUCJP
		; length		= strlen((char *)euc_string)
		; memcpy(str,	euc_string, length)
		; replace_code	= '@'
		; utf8str_size	= (2 * length)
		; memset(utf8str, (int)'\0', (utf8str_size + 1))
		; padding_code	= '\0'
		; how			= Sanitize_for_UTF8_Default_how
		; errno			= 0;
		;
		; fprintf(stderr,	"%s:L%d:%s(): to_utf8("
							"inbuf_encode=%d, inbuf@%p, inbuf_length=%lu"
							", replace_code='%c'(=0x%02x)"
							", outbuf@%p, outbuf_size=%lu"
							", padding_code='%c'(=0x%02x)"
							", how=0x%lx"
							") returns "
							, __FILE__, __LINE__, __func__
							, str_encoding, str, (unsigned long)length
							, replace_code, (unsigned)replace_code
							, utf8str, (unsigned long)utf8str_size
							, padding_code, (unsigned)padding_code
							, how
							); (void)fflush(stderr)
		; result = to_utf8(str_encoding
							, str
							, length
							, replace_code
							, utf8str
							, utf8str_size
							, padding_code
							, how
							)
		; sv_errno = errno; errmsg = strerror(sv_errno)
		; fprintf(stderr, "@%p(;en=%lu) (errno=%d: %s)\n"
							, result, (unsigned long)strlen((char *)result)
							, errno, errmsg
							); (void)fflush(stderr)
							;

		/* dump output of `to_utf8()' */
		fname	= "euc.to_utf8.txt";
		if ((fp = fopen(fname, "w")) == NULL) {
			sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr
				, "%s:L%d:%s(): Error! can't open %s (errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, fname, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		fprintf(fp, "%s", (char *)utf8str); (void)fflush(fp)
		; (void)fclose(fp)
		;
		fprintf(stderr, "%s:L%d:%s(): dumped EUC-JP `to_utf8()' data to "
						"\"%s\".\n"
						, __FILE__, __LINE__, __func__
						, fname
						); (void)fflush(stderr)
						;

		/* invoke from_utf8() (if to_utf8() returned normally) */
		if (result == utf8str) {
			str_encoding	= Sanitize_for_UTF8_e_EUCJP
			; utf8str_size	= strlen((char *)utf8str)
			; length		= strlen((char *)euc_string)
			; memset(str,	(int)'\0', length)
			; padding_code	= '$'
			; errno			= 0;
			;
			; fprintf(stderr,	"%s:L%d:%s(): from_utf8("
								"outbuf_encode=%d, inbuf@%p"
								", inbuf_length=%lu"
								", outbuf@%p, outbuf_size=%lu"
								", padding_code='%c'(=0x%02x)"
								") returns "
								, __FILE__, __LINE__, __func__
								, str_encoding, utf8str
								, (unsigned long)utf8str_size
								, str, (unsigned long)length
								, padding_code, (unsigned)padding_code
								); (void)fflush(stderr)
			; result = from_utf8(str_encoding
								, utf8str
								, utf8str_size
								, str
								, length
								, padding_code
								)
			; sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr, "@%p(len=%lu) (errno=%d: %s)\n"
								, result
								,	(unsigned long)strlen((char *)result)
								, errno, errmsg
								); (void)fflush(stderr)
								;

			/* dump output of `from_utf8()' */
			fname	= "euc.from_utf8.txt";
			if ((fp = fopen(fname, "w")) == NULL) {
				sv_errno = errno; errmsg = strerror(sv_errno)
				; fprintf(stderr
					, "%s:L%d:%s(): Error! can't open %s (errno=%d: %s)\n"
					, __FILE__, __LINE__, __func__
					, fname, sv_errno, errmsg
					); (void)fflush(stderr)
					;
				exit(EXIT_FAILURE);	/* Fatal exit */
			}
			fprintf(fp, "%s", (char *)str); (void)fflush(fp)
			; (void)fclose(fp)
			;
			fprintf(stderr, "%s:L%d:%s(): dumped EUC-JP `from_utf8()' "
							"data to \"%s\".\n"
							, __FILE__, __LINE__, __func__
							, fname
							); (void)fflush(stderr)
							;
		}

		/* release allocated resources */
		free(utf8str); utf8str = NULL;
		free(str); str = NULL;

		/* Check that "euc.sanitized.txt" equals "euc.from_utf8.txt"? */
		fprintf(stderr
			, "!!! %s:L%d:%s(): diff -c euc.sanitized.txt "
				"euc.from_utf8.txt\n"
			, __FILE__, __LINE__, __func__
			); (void)fflush(stderr)
		; (void)system("diff -c euc.sanitized.txt euc.from_utf8.txt")
		;
	}

	if (((ac == 1)
	|| strcmp(av[1], "--sjis") == 0) || strcmp(av[1], "-s") == 0) {
		unsigned char		*result = NULL;

		/* invoke from_utf8() (UTF-8 redundant code) */
		str_encoding	= Sanitize_for_UTF8_e_SJIS
		; utf8str_size	= strlen((char *)utf8_redundant_str)
		; length		= utf8str_size
		; if ((str = malloc((length + 1))) == NULL) {
			sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr
				, "%s:L%d:%s: Error! can't allocate str buffer "
					"(length+1=%lu)"
					"(errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, (unsigned long)(length + 1)
				, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		memset(str, (int)'\0', length)
		; padding_code	= '\0'
		; errno			= 0
		;
		fprintf(stderr, "%s:L%d:%s(): from_utf8(outbuf_encode=%d, "
						"inbuf@%p, inbuf_length=%lu, outbuf@%p, "
						"outbuf_size=%lu, padding_code='%c'(0x%02x)) "
						"returns "
						, __FILE__, __LINE__, __func__, str_encoding
						, utf8_redundant_str, (unsigned long)utf8str_size
						, str, (unsigned long)length
						, padding_code, (unsigned)padding_code
						); (void)fflush(stderr)
		; result	= from_utf8(str_encoding
								, utf8_redundant_str
								, utf8str_size
								, str
								, length
								, padding_code
								)
		; sv_errno = errno; errmsg = strerror(sv_errno)
		; fprintf(stderr, "%p (len=%lu) (errno=%d: %s)\n"
						, result, (unsigned long)strlen((char *)result)
						, sv_errno, errmsg
						); (void)fflush(stderr)
						;

		/* dump converted UTF-8 redundant code */
		fname	= "sjis.from_utf8.redundant.txt";
		if ((fp = fopen(fname, "w")) == NULL) {
			sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr
				, "%s:L%d:%s(): Error! can't open %s (errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, fname, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		fprintf(fp, "%s", (char *)str); (void)fflush(fp)
		; (void)fclose(fp)
		;
		fprintf(stderr, "%s:L%d:%s(): dumped converted UTF-8 redundant to "
						"SJIS in \"%s\".\n"
						, __FILE__, __LINE__, __func__
						, fname
						); (void)fflush(stderr)
						;

		/* release converted EUC-JP buffer */
		free(str); str = NULL;

		/* dump source text */
		fname	= "sjis.src.txt";
		if ((fp = fopen(fname, "w")) == NULL) {
			sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr
				, "%s:L%d:%s: Error! can't open %s (errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, fname, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		fprintf(fp, "%s", (char *)sjis_string); (void)fflush(fp)
		; (void)fclose(fp)
		;
		fprintf(stderr, "%s:L%d:%s(): dumped SJIS source data to \"%s\".\n"
						, __FILE__, __LINE__, __func__
						, fname
						); (void)fflush(stderr)
						;

		/* invoke `sanitize_for_utf8()' */
		length	= strlen((char *)sjis_string) + 1;
		if ((str = malloc((length + 1))) == NULL) {
			sv_errno	= errno;
			errmsg	= strerror(sv_errno);
			fprintf(stderr
				, "%s:L%d:%s: Error! can't allocate sjis_string buffer "
					"(errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		str_encoding	= Sanitize_for_UTF8_e_SJIS
		; length			= strlen((char *)sjis_string)
		; strcpy(memset(str, (int)'\0', (length + 1)), (char *)sjis_string)
		; replace_code	= '@'
		; fprintf(stderr, "%s:L%d:%s(): sanitize_for_utf8("
						"str_encoding=%d"
						", length=%lu (strlen(euc_string)=%lu)"
						", str@%p"
						", replace_code='%c'(=0x%02x)"
						") returns "
						, __FILE__, __LINE__, __func__
						, (int)str_encoding
						, (unsigned long)length
						,		(unsigned long)strlen((char *)euc_string)
						, (void *)str
						, replace_code, (unsigned)replace_code
						); (void)fflush(stderr)
						;
		errno	= 0
		; result	= sanitize_for_utf8(str_encoding
										, length
										, str
										, replace_code
										)
		; sv_errno	= errno; errmsg = strerror(sv_errno)
		;
		fprintf(stderr, "%p (errno=%d: %s).\n"
						, result, sv_errno, errmsg
						); (void)fflush(stderr)
						;

		/* dump output of `sanitize_for_utf8()' */
		fname	= "sjis.sanitized.txt";
		if ((fp = fopen(fname, "w")) == NULL) {
			sv_errno	= errno;
			errmsg	= strerror(sv_errno);
			fprintf(stderr
				, "%s:L%d:%s(): Error! can't open %s (errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, fname, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		fprintf(fp, "%s", (char *)str); (void)fflush(fp);
		(void)fclose(fp);
		fprintf(stderr, "%s:L%d:%s(): dumped SIS sanitized data to "
						"\"%s\".\n"
						, __FILE__, __LINE__, __func__
						, fname
						); (void)fflush(stderr)
						;

		/* invoke to_utf8() */
		str_encoding	= Sanitize_for_UTF8_e_SJIS
		; length		= strlen((char *)sjis_string)
		; memcpy(str,	sjis_string, length)
		; replace_code	= '@'
		; utf8str_size	= (2 * length)
		;
		if ((utf8str 	= malloc((utf8str_size + 1))) == NULL) {
			sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr
				, "%s:L%d:%s: Error! can't allocate utf8str buffer "
					"(utf8str_size+1=%lu)"
					"(errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, (unsigned long)(utf8str_size + 1)
				, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		memset(utf8str, (int)'\0', (utf8str_size + 1))
		; utf8str_size	= length
		; padding_code	= '\0'
		; how			= Sanitize_for_UTF8_Default_how
		; errno			= 0;
		;
		; fprintf(stderr,	"%s:L%d:%s(): to_utf8("
							"inbuf_encode=%d, inbuf@%p, inbuf_length=%lu"
							", replace_code='%c'(=0x%02x)"
							", outbuf@%p, outbuf_size=%lu"
							", padding_code='%c'(=0x%02x)"
							", how=0x%lx"
							") returns "
							, __FILE__, __LINE__, __func__
							, str_encoding, str, (unsigned long)length
							, replace_code, (unsigned)replace_code
							, utf8str, (unsigned long)utf8str_size
							, padding_code, (unsigned)padding_code
							, how
							); (void)fflush(stderr)
		; result = to_utf8(str_encoding
							, str
							, length
							, replace_code
							, utf8str
							, utf8str_size
							, padding_code
							, how
							)
		; sv_errno = errno; errmsg = strerror(sv_errno)
		; fprintf(stderr, "@%p(len=%lu) (errno=%d: %s)\n"
							, result, (unsigned long)strlen((char *)result)
							, errno, errmsg
							); (void)fflush(stderr)
							;

		/* dump output of `to_utf8()' (utf8str_size equals length) */
		fname	= "sjis.to_utf8.same_buff_size.txt";
		if ((fp = fopen(fname, "w")) == NULL) {
			sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr
				, "%s:L%d:%s(): Error! can't open %s (errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, fname, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		fprintf(fp, "%s", (char *)utf8str); (void)fflush(fp)
		; (void)fclose(fp)
		;
		fprintf(stderr, "%s:L%d:%s(): dumped SJIS `to_utf8( same buff "
						"size)' data to \"%s\".\n"
						, __FILE__, __LINE__, __func__
						, fname
						); (void)fflush(stderr)
						;

		str_encoding	= Sanitize_for_UTF8_e_SJIS
		; length		= strlen((char *)sjis_string)
		; memcpy(str,	sjis_string, length)
		; replace_code	= '@'
		; utf8str_size	= (2 * length)
		; memset(utf8str, (int)'\0', (utf8str_size + 1))
		; padding_code	= '\0'
		; how			= Sanitize_for_UTF8_Default_how
		; errno			= 0;
		;
		; fprintf(stderr,	"%s:L%d:%s(): to_utf8("
							"inbuf_encode=%d, inbuf@%p, inbuf_length=%lu"
							", replace_code='%c'(=0x%02x)"
							", outbuf@%p, outbuf_size=%lu"
							", padding_code='%c'(=0x%02x)"
							", how=0x%lx"
							") returns "
							, __FILE__, __LINE__, __func__
							, str_encoding, str, (unsigned long)length
							, replace_code, (unsigned)replace_code
							, utf8str, (unsigned long)utf8str_size
							, padding_code, (unsigned)padding_code
							, how
							); (void)fflush(stderr)
		; result = to_utf8(str_encoding
							, str
							, length
							, replace_code
							, utf8str
							, utf8str_size
							, padding_code
							, how
							)
		; sv_errno = errno; errmsg = strerror(sv_errno)
		; fprintf(stderr, "@%p(;en=%lu) (errno=%d: %s)\n"
							, result, (unsigned long)strlen((char *)result)
							, errno, errmsg
							); (void)fflush(stderr)
							;

		/* dump output of `to_utf8()' */
		fname	= "sjis.to_utf8.txt";
		if ((fp = fopen(fname, "w")) == NULL) {
			sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr
				, "%s:L%d:%s(): Error! can't open %s (errno=%d: %s)\n"
				, __FILE__, __LINE__, __func__
				, fname, sv_errno, errmsg
				); (void)fflush(stderr)
				;
			exit(EXIT_FAILURE);	/* Fatal exit */
		}
		fprintf(fp, "%s", (char *)utf8str); (void)fflush(fp)
		; (void)fclose(fp)
		;
		fprintf(stderr, "%s:L%d:%s(): dumped SJIS `to_utf8()' data to "
						"\"%s\".\n"
						, __FILE__, __LINE__, __func__
						, fname
						); (void)fflush(stderr)
						;

		/* invoke from_utf8() (if to_utf8() returned normally) */
		if (result == utf8str) {
			str_encoding	= Sanitize_for_UTF8_e_SJIS
			; utf8str_size	= strlen((char *)utf8str)
			; length		= strlen((char *)sjis_string)
			; memset(str,	(int)'\0', length)
			; padding_code	= '$'
			; errno			= 0;
			;
			; fprintf(stderr,	"%s:L%d:%s(): from_utf8("
								"outbuf_encode=%d, inbuf@%p"
								", inbuf_length=%lu"
								", outbuf@%p, outbuf_size=%lu"
								", padding_code='%c'(=0x%02x)"
								") returns "
								, __FILE__, __LINE__, __func__
								, str_encoding, utf8str
								, (unsigned long)utf8str_size
								, str, (unsigned long)length
								, padding_code, (unsigned)padding_code
								); (void)fflush(stderr)
			; result = from_utf8(str_encoding
								, utf8str
								, utf8str_size
								, str
								, length
								, padding_code
								)
			; sv_errno = errno; errmsg = strerror(sv_errno)
			; fprintf(stderr, "@%p(len=%lu) (errno=%d: %s)\n"
								, result
								,	(unsigned long)strlen((char *)result)
								, errno, errmsg
								); (void)fflush(stderr)
								;

			/* dump output of `from_utf8()' */
			fname	= "sjis.from_utf8.txt";
			if ((fp = fopen(fname, "w")) == NULL) {
				sv_errno = errno; errmsg = strerror(sv_errno)
				; fprintf(stderr
					, "%s:L%d:%s(): Error! can't open %s (errno=%d: %s)\n"
					, __FILE__, __LINE__, __func__
					, fname, sv_errno, errmsg
					); (void)fflush(stderr)
					;
				exit(EXIT_FAILURE);	/* Fatal exit */
			}
			fprintf(fp, "%s", (char *)str); (void)fflush(fp)
			; (void)fclose(fp)
			;
			fprintf(stderr, "%s:L%d:%s(): dumped SJIS `from_utf8()' data "
							"to \"%s\".\n"
							, __FILE__, __LINE__, __func__
							, fname
							); (void)fflush(stderr)
							;
		}

		/* release allocated resources */
		free(utf8str); utf8str = NULL;
		free(str); str = NULL;

		/* Check that "sjis.sanitized.txt" equals "sjis.from_utf8.txt"? */
		fprintf(stderr
			, "!!! %s:L%d:%s(): "
				"diff -c sjis.sanitized.txt sjis.from_utf8.txt"
				"\n"
			, __FILE__, __LINE__, __func__
			); (void)fflush(stderr)
		; (void)system("diff -c sjis.sanitized.txt sjis.from_utf8.txt")
		;
	}

	if (ac == 1) {
		/* Check different between "euc.to_utf8.txt" and "sjis.to_utf8.txt"
		 */
		fprintf(stderr, "!!! %s:L%d:%s(): "
						"diff -c euc.to_utf8.txt sjis.to_utf8.txt"
						"\n"
						, __FILE__, __LINE__, __func__
						); (void)fflush(stderr)
		; (void)system("diff -c euc.to_utf8.txt sjis.to_utf8.txt")
		;

		/* Check "euc.from_utf8.redundant.txt" equals
		 * "sjis.from_utf8.redundant.txt"?
		 */
		fprintf(stderr
			, "!!! %s:L%d:%s(): "
				"diff -c euc.from_utf8.redundant.txt "
				"sjis.from_utf8.redundant.txt"
				"\n"
			, __FILE__, __LINE__, __func__
			); (void)fflush(stderr)
		; (void)system("diff -c euc.from_utf8.redundant.txt "
						"sjis.from_utf8.redundant.txt"
						);
	}

	exit(EXIT_SUCCESS);	/* normal exit */
}
